/*
   Written 2002 by Josef Soucek <josef.soucek@ct.cz>
   Please read pclink.c for more information.
*/

#define _GNU_SOURCE
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <dirent.h>
#include <glob.h>
#include <sys/stat.h>
#include <sys/types.h>
#include "files.h"

#define true 1
#define false 0

static char *convertfname(char *,const char *);
static unsigned char ascii2petascii(unsigned char);

#define storetodir(x) dirbuff[dirpointer++]=(x)
//#define storetodir(x) putchar(x)

#define C64_BLOCK 254

static const char dirheader[]="\x01\x04\x01\x01\x00\x00\x12\"PC-LINK V0.1    \" 00 2A";
static const char dirfooter[]="BLOCKS FULL.             ";
static const char *c64_types[]={"PRG","DIR","<"," ","!","*"};
static char c64_line[30];            /* complete c64 dir line is 32 bytes */
static char c64_tmp[25];

int preparedirectory(char *dirbuff)
{
  int c64_tmplen;
  int c64_typenum,c64_protnum;
  int c64_blockslen;
  int c64_totalblocks;
  int dirpointer;
  int a;
  struct dirent *dir_struct;
  struct stat file_struct;
  char *dirname;
  DIR *dir;

  dirname=get_current_dir_name();

//  printf("pracovni adresar je: %s\n",dirname);

  if ((dir=opendir(dirname))==NULL)
  {
    printf("Opendir error...\n");
    exit(1);
  }

  free(dirname);                /* free the get_current_dir_name path */

  readdir(dir);                 /* "." current directory */

  dirpointer=0;

  for(a=0; a<32; a++) storetodir(dirheader[a]);

//  readdir(dir);                 /* ".." directory */

  c64_totalblocks=0;

  while((dir_struct=readdir(dir))!=NULL)
  {
    stat(dir_struct->d_name,&file_struct);

//    printf("File: %s Size: %d\n",dir_struct->d_name, file_struct.st_size);

    if (S_ISDIR(file_struct.st_mode)) c64_typenum=1;
    else c64_typenum=0;

    if (!(access(dir_struct->d_name, R_OK)))    /* Readable */
    {
      if (!(access(dir_struct->d_name, W_OK))) c64_protnum=3;
      else c64_protnum=2;

      c64_blockslen=file_struct.st_size / C64_BLOCK;
      if (file_struct.st_size % C64_BLOCK) c64_blockslen++;

      c64_totalblocks += c64_blockslen;

      sprintf(c64_tmp,"%d",c64_blockslen);
      c64_tmplen=strlen(c64_tmp);

//    fprintf(stderr,"!! %d %d\n",strlen(c64_tmp),c64_tmplen);

      c64_tmp[0]='\0';            /* clear the string */

      while((strlen(c64_tmp)+c64_tmplen) < 4)
        strcat(c64_tmp," ");
      strcat(c64_tmp,"\"");

      convertfname(c64_tmp,dir_struct->d_name);

      strcat(c64_tmp,"\"");

//    fprintf(stderr,"@@ %d %d\n",strlen(c64_tmp),c64_tmplen);

      while((strlen(c64_tmp)+c64_tmplen) < 23)
        strcat(c64_tmp," ");

//    fprintf(stderr,"## %d %d\n",strlen(c64_tmp),c64_tmplen);

      if (c64_blockslen > 0xffff)       /* Number of blocks owerflow "!" */
        c64_protnum=4;

      sprintf(c64_line,"%s%s%s",c64_tmp,c64_types[c64_typenum],c64_types[c64_protnum]);
      while(strlen(c64_line) < 27)
        strcat(c64_line," ");

      storetodir(0x1);
      storetodir(0x1);
      storetodir(c64_blockslen & 0xff);     /* low */
      storetodir(c64_blockslen >> 8);       /* hi  */

      c64_tmplen=strlen(c64_line);
      for(a=0; a<=c64_tmplen; a++) storetodir(c64_line[a]);
    }
  }
  storetodir(0x1);
  storetodir(0x1);
  storetodir(c64_totalblocks & 0xff);     /* low */
  storetodir(c64_totalblocks >> 8);       /* hi  */

//  if (c64_totalblocks > 0xffff)       /* Number of blocks owerflow "!" */

  c64_tmplen=strlen(dirfooter);
  for(a=0; a<=c64_tmplen; a++) storetodir(dirfooter[a]);
  storetodir(0x0);
  storetodir(0x0);

  closedir(dir);
  return dirpointer;
}

static unsigned char ascii2petascii(unsigned char data)
{
  /* Comment next line to list dir in lovercase chars */
  if ((data>=0x41) && (data<=0x5a))  return(data | 0x80);
  if ((data>=0x61) && (data<=0x7a))  return(data & 0xdf);
  return data;
}

static char *convertfname(char *destname,const char *srcname)
{
  int a;
  int srcsize,destsize;

  destsize=strlen(destname);
  if((srcsize=strlen(srcname))<=17)       /* 16 + '\0' */
  {
    for(a=0;a<srcsize;a++)
      destname[a+destsize]=ascii2petascii(srcname[a]);
    destname[a+destsize]='\0';
  }
  else
  {
    for(a=0;a<=14;a++)
      destname[a+destsize]=ascii2petascii(srcname[a]);
    destname[a+destsize]='*';
    destname[a+destsize+1]='\0';
  }
  return destname;
}

int deletefile(unsigned char *filename)
{
  if (remove(filename)) return false;
    return true;
}
int setnewdirpath(unsigned char *filename)
{

  if (!strlen(filename))
    strcpy(filename,"/");

  if (chdir(filename)) return false;
    return true;

    /* Nahradit / */
    /* Nahredit "" */
}

int makenewdirectory(unsigned char *filename)
{
  if (mkdir(filename,0755)) return false;
    return true;
}
int renamefile(unsigned char *filename)
{
  char *newname;
  char *oldname;

  /* newname=oldname */
  
  newname=strtok(filename,"=");
  oldname=strtok(NULL,"=");

  printf("Renaming file %s to %s",oldname,newname);

  if (rename(oldname, newname)) return false;
  return true;
}

int changeroot(unsigned char *dirname)
{
  char dirpath[256];

  if (dirname==NULL)
//    dirname=get_current_dir_name();
    getcwd(dirpath, 256);
  else
    strcpy(dirpath,dirname);

  if (chroot(dirpath))  /* Only superuser can change root path */
  {
//    free(dirname);
    printf("Change the root directory failed...\n");
    printf("Changing path to \"/\"\n");
    if (chdir("/")) return false;
      return true;
  }
  printf("Changed the root directory to %s\n",dirpath);
//  free(dirname);
  return true;
}

FILE *openfile(unsigned char *filename)
{
  glob_t globbuf;
  FILE *fr;

  glob(filename, 0, NULL, &globbuf);
  if (globbuf.gl_pathc)
  {
    fr=fopen(globbuf.gl_pathv[0],"rb");
    globfree(&globbuf);
    return fr;
  }
  else
  {
    printf("File %s not found error...\n",filename);
    globfree(&globbuf);
    return NULL;
  }
}

FILE *createfile(unsigned char *filename)
{
  FILE *fw;

  fw=fopen(filename,"wb");
  return fw;
}

